# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._apps_operations import build_create_app_request, build_list_apps_request, build_update_app_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class AppsOperations:
    """AppsOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~ignos.api.client.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace_async
    async def list_apps(
        self,
        **kwargs: Any
    ) -> List["_models.AppDto"]:
        """Internal Ignos api for listing apps.

        Internal Ignos api for listing apps.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of AppDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.AppDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.AppDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_apps_request(
            template_url=self.list_apps.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[AppDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_apps.metadata = {'url': '/admin/apps'}  # type: ignore


    @distributed_trace_async
    async def create_app(
        self,
        body: Optional["_models.CreateApp"] = None,
        **kwargs: Any
    ) -> "_models.AppDto":
        """Internal Ignos api for creating a new app.

        Internal Ignos api for creating a new app.

        :param body:
        :type body: ~ignos.api.client.models.CreateApp
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AppDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.AppDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.AppDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'CreateApp')
        else:
            _json = None

        request = build_create_app_request(
            content_type=content_type,
            json=_json,
            template_url=self.create_app.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('AppDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    create_app.metadata = {'url': '/admin/apps'}  # type: ignore


    @distributed_trace_async
    async def update_app(
        self,
        id: str,
        body: Optional["_models.UpdateAppRequest"] = None,
        **kwargs: Any
    ) -> "_models.AppDto":
        """Internal Ignos api for updating an app.

        Internal Ignos api for updating an app.

        :param id:
        :type id: str
        :param body:
        :type body: ~ignos.api.client.models.UpdateAppRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AppDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.AppDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.AppDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'UpdateAppRequest')
        else:
            _json = None

        request = build_update_app_request(
            id=id,
            content_type=content_type,
            json=_json,
            template_url=self.update_app.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('AppDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_app.metadata = {'url': '/admin/apps/{id}'}  # type: ignore

