# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._cdf_clusters_operations import build_create_cdf_cluster_request, build_list_cdf_clusters_request, build_update_cdf_cluster_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class CdfClustersOperations:
    """CdfClustersOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~ignos.api.client.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace_async
    async def list_cdf_clusters(
        self,
        **kwargs: Any
    ) -> List["_models.CdfClusterDto"]:
        """list_cdf_clusters.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of CdfClusterDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.CdfClusterDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.CdfClusterDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_cdf_clusters_request(
            template_url=self.list_cdf_clusters.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[CdfClusterDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_cdf_clusters.metadata = {'url': '/admin/cdfclusters'}  # type: ignore


    @distributed_trace_async
    async def create_cdf_cluster(
        self,
        body: Optional["_models.CreateCdfCluster"] = None,
        **kwargs: Any
    ) -> "_models.CdfClusterDto":
        """create_cdf_cluster.

        :param body:
        :type body: ~ignos.api.client.models.CreateCdfCluster
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CdfClusterDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.CdfClusterDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.CdfClusterDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'CreateCdfCluster')
        else:
            _json = None

        request = build_create_cdf_cluster_request(
            content_type=content_type,
            json=_json,
            template_url=self.create_cdf_cluster.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('CdfClusterDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    create_cdf_cluster.metadata = {'url': '/admin/cdfclusters'}  # type: ignore


    @distributed_trace_async
    async def update_cdf_cluster(
        self,
        id: str,
        body: Optional["_models.UpdateCdfClusterRequest"] = None,
        **kwargs: Any
    ) -> "_models.CdfClusterDto":
        """update_cdf_cluster.

        :param id:
        :type id: str
        :param body:
        :type body: ~ignos.api.client.models.UpdateCdfClusterRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CdfClusterDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.CdfClusterDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.CdfClusterDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'UpdateCdfClusterRequest')
        else:
            _json = None

        request = build_update_cdf_cluster_request(
            id=id,
            content_type=content_type,
            json=_json,
            template_url=self.update_cdf_cluster.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('CdfClusterDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_cdf_cluster.metadata = {'url': '/admin/cdfclusters/{id}'}  # type: ignore

