# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import Any, Callable, Dict, Generic, Optional, TypeVar
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._customer_orders_operations import build_upsert_customer_order_line_request, build_upsert_customer_order_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class CustomerOrdersOperations:
    """CustomerOrdersOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~ignos.api.client.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace_async
    async def upsert_customer_order(
        self,
        id: str,
        body: Optional["_models.UpsertCustomerOrderRequest"] = None,
        **kwargs: Any
    ) -> None:
        """Upsert a customer order.

        Upsert a customer order.

        :param id: Customer order id.
        :type id: str
        :param body: Order details.
        :type body: ~ignos.api.client.models.UpsertCustomerOrderRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'UpsertCustomerOrderRequest')
        else:
            _json = None

        request = build_upsert_customer_order_request(
            id=id,
            content_type=content_type,
            json=_json,
            template_url=self.upsert_customer_order.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    upsert_customer_order.metadata = {'url': '/erp/customerorders/{id}'}  # type: ignore


    @distributed_trace_async
    async def upsert_customer_order_line(
        self,
        id: str,
        line: int,
        body: Optional["_models.CustomerOrderLineDto"] = None,
        **kwargs: Any
    ) -> None:
        """Upsert a customer order line. Customer order must exist.

        Upsert a customer order line. Customer order must exist.

        :param id: Customer order id.
        :type id: str
        :param line: Customer order line.
        :type line: int
        :param body: Order line details.
        :type body: ~ignos.api.client.models.CustomerOrderLineDto
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'CustomerOrderLineDto')
        else:
            _json = None

        request = build_upsert_customer_order_line_request(
            id=id,
            line=line,
            content_type=content_type,
            json=_json,
            template_url=self.upsert_customer_order_line.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    upsert_customer_order_line.metadata = {'url': '/erp/customerorders/{id}/orderlines/{line}'}  # type: ignore

