# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._customers_operations import build_add_customer_app_request, build_create_customer_request, build_create_tenant_request, build_get_customer_request, build_list_customer_apps_request, build_list_customer_tenants_request, build_list_customers_request, build_list_tenant_keys_request, build_remove_customer_app_request, build_update_customer_request, build_update_tenant_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class CustomersOperations:
    """CustomersOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~ignos.api.client.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace_async
    async def list_customers(
        self,
        page_size: Optional[int] = 100,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> List["_models.IgnosCustomerDto"]:
        """Internal Ignos api for listing customers.

        Internal Ignos api for listing customers.

        :param page_size:
        :type page_size: int
        :param continuation_token_parameter:
        :type continuation_token_parameter: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of IgnosCustomerDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.IgnosCustomerDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.IgnosCustomerDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_customers_request(
            page_size=page_size,
            continuation_token_parameter=continuation_token_parameter,
            template_url=self.list_customers.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[IgnosCustomerDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_customers.metadata = {'url': '/admin/customers'}  # type: ignore


    @distributed_trace_async
    async def create_customer(
        self,
        body: Optional["_models.CreateIgnosCustomer"] = None,
        **kwargs: Any
    ) -> "_models.IgnosCustomerDto":
        """Internal Ignos api for creating a customer.

        Internal Ignos api for creating a customer.

        :param body:
        :type body: ~ignos.api.client.models.CreateIgnosCustomer
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: IgnosCustomerDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.IgnosCustomerDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.IgnosCustomerDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'CreateIgnosCustomer')
        else:
            _json = None

        request = build_create_customer_request(
            content_type=content_type,
            json=_json,
            template_url=self.create_customer.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('IgnosCustomerDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    create_customer.metadata = {'url': '/admin/customers'}  # type: ignore


    @distributed_trace_async
    async def get_customer(
        self,
        id: str,
        **kwargs: Any
    ) -> "_models.IgnosCustomerDto":
        """Internal Ignos api for getting a customer.

        Internal Ignos api for getting a customer.

        :param id:
        :type id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: IgnosCustomerDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.IgnosCustomerDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.IgnosCustomerDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_get_customer_request(
            id=id,
            template_url=self.get_customer.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('IgnosCustomerDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_customer.metadata = {'url': '/admin/customers/{id}'}  # type: ignore


    @distributed_trace_async
    async def update_customer(
        self,
        id: str,
        body: Optional["_models.UpdateCustomerRequest"] = None,
        **kwargs: Any
    ) -> "_models.IgnosCustomerDto":
        """Internal Ignos api for updating a customer.

        Internal Ignos api for updating a customer.

        :param id:
        :type id: str
        :param body:
        :type body: ~ignos.api.client.models.UpdateCustomerRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: IgnosCustomerDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.IgnosCustomerDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.IgnosCustomerDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'UpdateCustomerRequest')
        else:
            _json = None

        request = build_update_customer_request(
            id=id,
            content_type=content_type,
            json=_json,
            template_url=self.update_customer.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('IgnosCustomerDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_customer.metadata = {'url': '/admin/customers/{id}'}  # type: ignore


    @distributed_trace_async
    async def list_customer_tenants(
        self,
        id: str,
        **kwargs: Any
    ) -> List["_models.TenantDto"]:
        """Internal Ignos api for a customer's tenants.

        Internal Ignos api for a customer's tenants.

        :param id:
        :type id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of TenantDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.TenantDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.TenantDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_customer_tenants_request(
            id=id,
            template_url=self.list_customer_tenants.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[TenantDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_customer_tenants.metadata = {'url': '/admin/customers/{id}/tenants'}  # type: ignore


    @distributed_trace_async
    async def create_tenant(
        self,
        id: str,
        body: Optional["_models.CreateTenantRequest"] = None,
        **kwargs: Any
    ) -> "_models.TenantDto":
        """Internal Ignos api for creating a customer tenant.

        Internal Ignos api for creating a customer tenant.

        :param id:
        :type id: str
        :param body:
        :type body: ~ignos.api.client.models.CreateTenantRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: TenantDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.TenantDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.TenantDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'CreateTenantRequest')
        else:
            _json = None

        request = build_create_tenant_request(
            id=id,
            content_type=content_type,
            json=_json,
            template_url=self.create_tenant.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('TenantDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    create_tenant.metadata = {'url': '/admin/customers/{id}/tenants'}  # type: ignore


    @distributed_trace_async
    async def update_tenant(
        self,
        id: str,
        tenant_id: str,
        body: Optional["_models.UpdateTenantRequest"] = None,
        **kwargs: Any
    ) -> "_models.TenantDto":
        """Internal Ignos api for updating a customer tenant.

        Internal Ignos api for updating a customer tenant.

        :param id:
        :type id: str
        :param tenant_id:
        :type tenant_id: str
        :param body:
        :type body: ~ignos.api.client.models.UpdateTenantRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: TenantDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.TenantDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.TenantDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'UpdateTenantRequest')
        else:
            _json = None

        request = build_update_tenant_request(
            id=id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            template_url=self.update_tenant.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('TenantDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_tenant.metadata = {'url': '/admin/customers/{id}/tenants/{tenantId}'}  # type: ignore


    @distributed_trace_async
    async def list_customer_apps(
        self,
        id: str,
        **kwargs: Any
    ) -> List["_models.CustomerAppDto"]:
        """Internal Ignos api for listing customer apps.

        Internal Ignos api for listing customer apps.

        :param id:
        :type id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of CustomerAppDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.CustomerAppDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.CustomerAppDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_customer_apps_request(
            id=id,
            template_url=self.list_customer_apps.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[CustomerAppDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_customer_apps.metadata = {'url': '/admin/customers/{id}/apps'}  # type: ignore


    @distributed_trace_async
    async def add_customer_app(
        self,
        id: str,
        app_id: str,
        **kwargs: Any
    ) -> None:
        """Internal Ignos api for adding an app to a customer.

        Internal Ignos api for adding an app to a customer.

        :param id:
        :type id: str
        :param app_id:
        :type app_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_add_customer_app_request(
            id=id,
            app_id=app_id,
            template_url=self.add_customer_app.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    add_customer_app.metadata = {'url': '/admin/customers/{id}/apps/{appId}'}  # type: ignore


    @distributed_trace_async
    async def remove_customer_app(
        self,
        id: str,
        app_id: str,
        **kwargs: Any
    ) -> None:
        """Internal Ignos api for removing an app from a customer.

        Internal Ignos api for removing an app from a customer.

        :param id:
        :type id: str
        :param app_id:
        :type app_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_remove_customer_app_request(
            id=id,
            app_id=app_id,
            template_url=self.remove_customer_app.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    remove_customer_app.metadata = {'url': '/admin/customers/{id}/apps/{appId}'}  # type: ignore


    @distributed_trace_async
    async def list_tenant_keys(
        self,
        **kwargs: Any
    ) -> List["_models.TenantKeyDto"]:
        """list_tenant_keys.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of TenantKeyDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.TenantKeyDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.TenantKeyDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_tenant_keys_request(
            template_url=self.list_tenant_keys.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[TenantKeyDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_tenant_keys.metadata = {'url': '/admin/customers/tenantkeys'}  # type: ignore

