# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._external_access_operations import build_create_company_user_request, build_delete_company_customer_request, build_delete_company_user_request, build_list_company_customers_request, build_list_roles_request, build_list_users_request, build_update_company_user_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class ExternalAccessOperations:
    """ExternalAccessOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~ignos.api.client.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace_async
    async def list_users(
        self,
        **kwargs: Any
    ) -> List["_models.CompanyUserDto"]:
        """list_users.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of CompanyUserDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.CompanyUserDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.CompanyUserDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_users_request(
            template_url=self.list_users.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[CompanyUserDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_users.metadata = {'url': '/externalaccess/users'}  # type: ignore


    @distributed_trace_async
    async def create_company_user(
        self,
        body: Optional["_models.CreateCompanyUserRequest"] = None,
        **kwargs: Any
    ) -> "_models.CompanyUserDto":
        """create_company_user.

        :param body:
        :type body: ~ignos.api.client.models.CreateCompanyUserRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CompanyUserDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.CompanyUserDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'CreateCompanyUserRequest')
        else:
            _json = None

        request = build_create_company_user_request(
            content_type=content_type,
            json=_json,
            template_url=self.create_company_user.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('CompanyUserDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    create_company_user.metadata = {'url': '/externalaccess/users'}  # type: ignore


    @distributed_trace_async
    async def update_company_user(
        self,
        id: str,
        body: Optional["_models.UpdateCompanyUserRequest"] = None,
        **kwargs: Any
    ) -> "_models.CompanyUserDto":
        """update_company_user.

        :param id:
        :type id: str
        :param body:
        :type body: ~ignos.api.client.models.UpdateCompanyUserRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CompanyUserDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.CompanyUserDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'UpdateCompanyUserRequest')
        else:
            _json = None

        request = build_update_company_user_request(
            id=id,
            content_type=content_type,
            json=_json,
            template_url=self.update_company_user.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('CompanyUserDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_company_user.metadata = {'url': '/externalaccess/users/{id}'}  # type: ignore


    @distributed_trace_async
    async def delete_company_user(
        self,
        id: str,
        **kwargs: Any
    ) -> None:
        """delete_company_user.

        :param id:
        :type id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_delete_company_user_request(
            id=id,
            template_url=self.delete_company_user.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    delete_company_user.metadata = {'url': '/externalaccess/users/{id}'}  # type: ignore


    @distributed_trace_async
    async def list_roles(
        self,
        **kwargs: Any
    ) -> List["_models.ExternalRoleDto"]:
        """list_roles.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of ExternalRoleDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.ExternalRoleDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.ExternalRoleDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_roles_request(
            template_url=self.list_roles.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[ExternalRoleDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_roles.metadata = {'url': '/externalaccess/roles'}  # type: ignore


    @distributed_trace_async
    async def list_company_customers(
        self,
        **kwargs: Any
    ) -> List["_models.CompanyCustomerDto"]:
        """list_company_customers.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of CompanyCustomerDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.CompanyCustomerDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.CompanyCustomerDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_company_customers_request(
            template_url=self.list_company_customers.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[CompanyCustomerDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_company_customers.metadata = {'url': '/externalaccess/customers'}  # type: ignore


    @distributed_trace_async
    async def delete_company_customer(
        self,
        tenant_id: str,
        **kwargs: Any
    ) -> None:
        """delete_company_customer.

        :param tenant_id:
        :type tenant_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_delete_company_customer_request(
            tenant_id=tenant_id,
            template_url=self.delete_company_customer.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    delete_company_customer.metadata = {'url': '/externalaccess/customers/{tenantId}'}  # type: ignore

