# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import datetime
import functools
from typing import TYPE_CHECKING
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from msrest import Serializer

from .. import models as _models
from .._vendor import _convert_request, _format_url_section

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False
# fmt: off

def build_list_machine_alarms_request(
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    asset_id = kwargs.pop('asset_id', None)  # type: Optional[int]
    start_time = kwargs.pop('start_time', None)  # type: Optional[datetime.datetime]
    end_time = kwargs.pop('end_time', None)  # type: Optional[datetime.datetime]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/machinealarms')

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if asset_id is not None:
        query_parameters['assetId'] = _SERIALIZER.query("asset_id", asset_id, 'long')
    if start_time is not None:
        query_parameters['startTime'] = _SERIALIZER.query("start_time", start_time, 'iso-8601')
    if end_time is not None:
        query_parameters['endTime'] = _SERIALIZER.query("end_time", end_time, 'iso-8601')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_list_alarms_per_machine_request(
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    start_time = kwargs.pop('start_time', None)  # type: Optional[datetime.datetime]
    end_time = kwargs.pop('end_time', None)  # type: Optional[datetime.datetime]
    asset_id = kwargs.pop('asset_id', None)  # type: Optional[int]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/machinealarms/alarmspermachine')

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if start_time is not None:
        query_parameters['startTime'] = _SERIALIZER.query("start_time", start_time, 'iso-8601')
    if end_time is not None:
        query_parameters['endTime'] = _SERIALIZER.query("end_time", end_time, 'iso-8601')
    if asset_id is not None:
        query_parameters['assetId'] = _SERIALIZER.query("asset_id", asset_id, 'long')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_list_alarm_severity_occurrences_request(
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    start_time = kwargs.pop('start_time', None)  # type: Optional[datetime.datetime]
    end_time = kwargs.pop('end_time', None)  # type: Optional[datetime.datetime]
    asset_id = kwargs.pop('asset_id', None)  # type: Optional[int]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/machinealarms/alarmtypeseverities')

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if start_time is not None:
        query_parameters['startTime'] = _SERIALIZER.query("start_time", start_time, 'iso-8601')
    if end_time is not None:
        query_parameters['endTime'] = _SERIALIZER.query("end_time", end_time, 'iso-8601')
    if asset_id is not None:
        query_parameters['assetId'] = _SERIALIZER.query("asset_id", asset_id, 'long')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_get_machine_alarm_details_request(
    id,  # type: int
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/machinealarms/alarms/{id}')
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, 'long'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        headers=header_parameters,
        **kwargs
    )

# fmt: on
class MachineAlarmsOperations(object):
    """MachineAlarmsOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~ignos.api.client.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def list_machine_alarms(
        self,
        asset_id=None,  # type: Optional[int]
        start_time=None,  # type: Optional[datetime.datetime]
        end_time=None,  # type: Optional[datetime.datetime]
        **kwargs  # type: Any
    ):
        # type: (...) -> List["_models.MachineAlarmDto"]
        """list_machine_alarms.

        :param asset_id:
        :type asset_id: long
        :param start_time:
        :type start_time: ~datetime.datetime
        :param end_time:
        :type end_time: ~datetime.datetime
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of MachineAlarmDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.MachineAlarmDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.MachineAlarmDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_machine_alarms_request(
            asset_id=asset_id,
            start_time=start_time,
            end_time=end_time,
            template_url=self.list_machine_alarms.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[MachineAlarmDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_machine_alarms.metadata = {'url': '/machinealarms'}  # type: ignore


    @distributed_trace
    def list_alarms_per_machine(
        self,
        start_time=None,  # type: Optional[datetime.datetime]
        end_time=None,  # type: Optional[datetime.datetime]
        asset_id=None,  # type: Optional[int]
        **kwargs  # type: Any
    ):
        # type: (...) -> List["_models.MachineAlarmSummaryDto"]
        """list_alarms_per_machine.

        :param start_time:
        :type start_time: ~datetime.datetime
        :param end_time:
        :type end_time: ~datetime.datetime
        :param asset_id:
        :type asset_id: long
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of MachineAlarmSummaryDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.MachineAlarmSummaryDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.MachineAlarmSummaryDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_alarms_per_machine_request(
            start_time=start_time,
            end_time=end_time,
            asset_id=asset_id,
            template_url=self.list_alarms_per_machine.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[MachineAlarmSummaryDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_alarms_per_machine.metadata = {'url': '/machinealarms/alarmspermachine'}  # type: ignore


    @distributed_trace
    def list_alarm_severity_occurrences(
        self,
        start_time=None,  # type: Optional[datetime.datetime]
        end_time=None,  # type: Optional[datetime.datetime]
        asset_id=None,  # type: Optional[int]
        **kwargs  # type: Any
    ):
        # type: (...) -> List["_models.MachineAlarmSeverityOccurenceDto"]
        """list_alarm_severity_occurrences.

        :param start_time:
        :type start_time: ~datetime.datetime
        :param end_time:
        :type end_time: ~datetime.datetime
        :param asset_id:
        :type asset_id: long
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of MachineAlarmSeverityOccurenceDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.MachineAlarmSeverityOccurenceDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[List["_models.MachineAlarmSeverityOccurenceDto"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_list_alarm_severity_occurrences_request(
            start_time=start_time,
            end_time=end_time,
            asset_id=asset_id,
            template_url=self.list_alarm_severity_occurrences.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[MachineAlarmSeverityOccurenceDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_alarm_severity_occurrences.metadata = {'url': '/machinealarms/alarmtypeseverities'}  # type: ignore


    @distributed_trace
    def get_machine_alarm_details(
        self,
        id,  # type: int
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.MachineAlarmDetailsDto"
        """get_machine_alarm_details.

        :param id:
        :type id: long
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: MachineAlarmDetailsDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.MachineAlarmDetailsDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.MachineAlarmDetailsDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_get_machine_alarm_details_request(
            id=id,
            template_url=self.get_machine_alarm_details.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('MachineAlarmDetailsDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_machine_alarm_details.metadata = {'url': '/machinealarms/alarms/{id}'}  # type: ignore

